import { world, ItemStack, MinecraftBlockTypes } from "@minecraft/server";
import { addToGlobalList } from "../global_list/list_functions";

const [x_location,y_location,z_location] = [0, -64, 0];


export function getPlayerByName(playerName) {
    return Array.from(world.getPlayers()).find(
        (player) => player.name === playerName
    );
}

export function getPlayerNameByObject(player) {
    return player.name;
}



export function saveDataAsBookLore(data, isCompleted, player) {

    const block = world.getDimension("overworld").getBlock({ x: x_location, y: y_location, z: z_location });

    const procBlocks = [
        world.getDimension("overworld").getBlock({ x: x_location+1, y: y_location, z: z_location }),
        world.getDimension("overworld").getBlock({ x: x_location-1, y: y_location, z: z_location }),
        world.getDimension("overworld").getBlock({ x: x_location, y: y_location, z: z_location+1 }),
        world.getDimension("overworld").getBlock({ x: x_location, y: y_location, z: z_location-1 }),
        world.getDimension("overworld").getBlock({ x: x_location, y: y_location+1, z: z_location })
    ];


    let name = getPlayerNameByObject(player);
    let newPlayer = true;
    let book = undefined;
    let slot = 0;
    let skipCheck = false;
    let spaceavailable = false;

    if (block.type != MinecraftBlockTypes.chest) {
        block.setType(MinecraftBlockTypes.chest);
        for(let i = 0;i< procBlocks.length; i++)
        {
            procBlocks[i].setType(MinecraftBlockTypes.bedrock);
        }
        skipCheck = true;
    }

    const inventoryComponent = block.getComponent("inventory");
    const inventoryContainer = inventoryComponent.container;
    const chestsize = 26;

    for(let i = 0;i< chestsize ;i++)
    {
        if(skipCheck){
            break;
        }
        
        let item = inventoryContainer.getItem(i);
        if(item && item.typeId == "minecraft:book"){
            let lore = item.getLore();
            let [key, playername] = lore[0].split("-");
            if(playername == name){
                newPlayer = false;
                slot = i;
                spaceavailable = true;
                break;
            }
        }
        else{
            slot = i;
            spaceavailable = true;
            break;
        }
    }

    if(!spaceavailable){
        player.sendMessage("TO-DO List is full, contact admin for this issue.");
    }

    if(newPlayer){
        book = new ItemStack("minecraft:book", 1);
        book.setLore(["Player-" + name, String(isCompleted) + "-" + data]);
    }
    else{
        book = inventoryContainer.getItem(slot);
        let lore = book.getLore();
        lore.push(String(isCompleted) + "-" + data);
        book.setLore(lore);
    }
    inventoryContainer.setItem(slot, book);
}


export function updateBookLore(player, task, index, isCompleted, changed) {

    const chest = world.getDimension("overworld").getBlock({ x: x_location, y: y_location, z: z_location });
    let book = undefined;
    
    if (chest.type != MinecraftBlockTypes.chest) {
        return;
    }

    const inventoryComponent = chest.getComponent("inventory");
    const inventoryContainer = inventoryComponent.container;
    const chestsize = 26;
    let name = getPlayerNameByObject(player);


    for(let i = 0;i< chestsize ;i++)
    {
        let item = inventoryContainer.getItem(i);
        if(item && item.typeId == "minecraft:book"){
            let lore = item.getLore();
            let [key, playername] = lore[0].split("-");
            if(playername == name){

                let tasks = [];
                let completed = [];
                let newlore = ["Player-"+playername];

                for(let j = 1;j<lore.length; j++)
                {
                    let [isChanged, oldtask] = lore[j].split("-");
                    isChanged = (isChanged == "true")?true:false;
                    if(isChanged)
                        completed.push(oldtask);
                    else
                        tasks.push(oldtask);
                }

                if(changed){
                    if(isCompleted){
                        tasks.splice(index,1);
                        completed.push(task);
                    }
                    else{
                        completed.splice(index,1);
                        tasks.push(task);
                    }
                }
                else{
                    if(isCompleted){
                        completed[index] = task;
                    }
                    else{
                        tasks[index] = task;
                    }
                }

                for(let j = 0;j<tasks.length; j++){
                    tasks[j] = "false-"+tasks[j];
                }
                for(let j = 0;j<completed.length; j++){
                    completed[j] = "true-"+completed[j];
                }

                newlore = newlore.concat(tasks,completed);
                item.setLore(newlore);
                inventoryContainer.setItem(i, item);
                break;
            }
        }
    }
}


export function deleteBookLore(player, task, index, isCompleted) {

    const chest = world.getDimension("overworld").getBlock({ x: x_location, y: y_location, z: z_location });
    let book = undefined;
    
    if (chest.type != MinecraftBlockTypes.chest) {
        return;
    }

    const inventoryComponent = chest.getComponent("inventory");
    const inventoryContainer = inventoryComponent.container;
    const chestsize = 26;
    let name = getPlayerNameByObject(player);


    for(let i = 0;i< chestsize ;i++)
    {
        let item = inventoryContainer.getItem(i);
        if(item && item.typeId == "minecraft:book"){
            let lore = item.getLore();
            let [key, playername] = lore[0].split("-");
            if(playername == name){

                let tasks = [];
                let completed = [];
                let newlore = ["Player-"+playername];

                for(let j = 1;j<lore.length; j++)
                {
                    let [isChanged, oldtask] = lore[j].split("-");
                    isChanged = (isChanged == "true")?true:false;
                    if(isChanged)
                        completed.push(oldtask);
                    else
                        tasks.push(oldtask);
                }

                if(isCompleted){
                    if(completed[index] == task)
                        completed.splice(index,1);
                }
                else{
                    if(tasks[index] == task)
                        tasks.splice(index,1);
                }

                for(let j = 0;j<tasks.length; j++){
                    tasks[j] = "false-"+tasks[j];
                }
                for(let j = 0;j<completed.length; j++){
                    completed[j] = "true-"+completed[j];
                }

                newlore = newlore.concat(tasks,completed);
                item.setLore(newlore);
                inventoryContainer.setItem(i, item);
                break;
            }
        }
    }
}


export function retrieveDataFromLore(GLOBALLIST)
{
    const chest = world.getDimension("overworld").getBlock({ x: x_location, y: y_location, z: z_location });
    let book = undefined;
    
    if (chest.type != MinecraftBlockTypes.chest) {
        return;
    }

    const inventoryComponent = chest.getComponent("inventory");
    const inventoryContainer = inventoryComponent.container;
    const chestsize = 26;


    for(let i = 0;i< chestsize ;i++)
    {
        let item = inventoryContainer.getItem(i);
        if(item && item.typeId == "minecraft:book"){
            let lore = item.getLore();
            let first = true;
            let player = undefined;
            lore.forEach((line)=>{
                if(first)
                {
                    let [key, playername] = line.split("-");
                    first = false;
                    player = getPlayerByName(playername);
                }
                else{
                    let [status, task] = line.split("-");
                    if(status == "true")
                        status = true;
                    else
                        status = false;
                    addToGlobalList(GLOBALLIST, player, task, status);
                }
            });
        }
    }
}
