import { ModalFormData, ActionFormData, MessageFormData } from "@minecraft/server-ui";
import { addToGlobalList, removeFromGlobalList, updateGlobalList } from "../global_list/list_functions.js"
import { saveDataAsBookLore, updateBookLore, deleteBookLore } from "../book_lore/lore_functions.js";


export function openTODOList(player, GLOBALLIST) {
    const messageForm = new MessageFormData()
        .title("§k--------§r§l§5]-ToDo List-[§r§k--------")
        .body("A §l§cTo-Do List§r is a tool for organizing tasks and responsibilities, helping individuals track and prioritize their actions to enhance productivity and goal achievement.\n\n-§l§cAdd Task §r- Add new task to your todo list.\n-§l§cView Tasks §r- Opens your todo list.")
        .button1("Add Task")
        .button2("View Tasks");

    messageForm
        .show(player)
        .then((formData) => {
            if (formData.canceled || formData.selection === undefined) {
                return;
            }

            if (formData.selection == 0) {
                show_list_entry_form(player, GLOBALLIST);
            }
            else if (formData.selection == 1) {
                show_todo_list(player, GLOBALLIST);
            }
        })
}


function show_todo_list(player, GLOBALLIST) {
    const playerData = GLOBALLIST.find((item) => item.player === player);
    if (!playerData || (playerData.tasks.length === 0 && playerData.completed.length === 0)) {
        display_list_is_empty(player);
    }
    else {
        const { tasks, completed } = playerData;

        const form = new ActionFormData().title("§k--------§r§l§5]-ToDo List-[§r§k--------");

        tasks.forEach((task) => {
            form.button("§l§b"+task + " §f§c[-Pending-]", "textures/items/cross.png");
        });

        completed.forEach((task) => {
            form.button("§l§b" + task + " §f§a[-Completed-]", "textures/items/tick.png");
        });

        form.show(player).then((response) => {
            if (response.canceled || response.selection == undefined) {
                return;
            }

            let task = undefined;
            let index = Number(response.selection);
            let status = false;

            if (index >= tasks.length) {
                task = completed[index - tasks.length];
                index = index - tasks.length;
                status = true;
            }
            else
                task = tasks[index];
            open_task(player, task, index, status, GLOBALLIST);

        });
    }
}


function show_list_entry_form(player, GLOBALLIST) {
    const formData = new ModalFormData();
    formData
        .title("§k--------§r§l§5]-ToDo List-[§r§k--------")
        .textField("Enter Task", "Enter your task details here...")
        .toggle("Mark as Completed")
        .show(player)
        .then((response) => {
            if (response.canceled) {
                return;
            }
            const task = response.formValues[0];
            if (task == "") {
                player.sendMessage("§l§c[!] Task cannot be empty!");
                return;
            }
            const isCompleted = response.formValues[1];

            addToGlobalList(GLOBALLIST, player, task, isCompleted);

            saveDataAsBookLore(task, isCompleted, player);

            if (task) {
                player.sendMessage(`Task added: §a§l${task}`);
            }
        });
}


function display_list_is_empty(player) {
    const messageForm = new MessageFormData()
        .title("§k-----§r§l§5]-Empty ToDo List-[§r§k-----")
        .body("Feel free to enhance your productivity by adding tasks to your To-Do List. \nIt's currently empty, so adding tasks will allow you to view and manage your list efficiently.\n\n§l§cNote : §rIf this dialogue shows up even if you have tasks in the list, ask the admin to run the command \"§l/reload§r\"!")
        .button1("Add Task in the List")
        .button2("Cancel");

    messageForm
        .show(player)
        .then((formData) => {
            if (formData.canceled || formData.selection === undefined) {
                return;
            }

            if (formData.selection == 0) {
                show_list_entry_form(player);
            }
            else if (formData.selection == 1) {
                return;
            }
        })
}


function open_task(player, task, index, status, GLOBALLIST) {
    let state = (status) ? "Completed" : "Pending";
    const messageForm = new MessageFormData()
        .title(`§l§7Task ${index + 1} of ${state} Tasks`)
        .body(`§lTask Details :\n§r${task}\n\n§lStatus : §r${state}`)
        .button1("Back")
        .button2("Edit Task");

    messageForm
        .show(player)
        .then((formData) => {
            if (formData.canceled || formData.selection === undefined) {
                return;
            }
            if (formData.selection == 0)
                show_todo_list(player, GLOBALLIST);
            if (formData.selection == 1)
                edit_task(player, index, task, status, GLOBALLIST);
        })
}


function edit_task(player, index, task, status, GLOBALLIST) {
    let state = (status) ? "Completed" : "Pending";
    const messageForm = new MessageFormData()
        .title(`§l§7Task ${index + 1} of ${state} Tasks`)
        .body(`§lTask Details :\n§r${task}\n\n§lStatus : §r${state}`)
        .button1("Delete Task")
        .button2("Edit Task Content/Task Status");

    messageForm
        .show(player)
        .then((formData) => {
            if (formData.canceled || formData.selection === undefined) {
                return;
            }
            if (formData.selection == 1)
                edit_task_details(GLOBALLIST, player, index, task, status);
            else if (formData.selection == 0)
                delete_task(player, index, task, status, GLOBALLIST);
        })
}


function edit_task_details(GLOBALLIST, player, index, task, status) {
    const formData = new ModalFormData();
    let changed = false;
    formData
        .title("§l§7Edit Task")
        .textField("Update Task", "Enter your task here...", task)
        .toggle("Mark as Completed", status)
        .show(player)
        .then((response) => {
            if (response.canceled) {
                return;
            }
            const task = response.formValues[0];
            if (task == "") {
                player.sendMessage("§l§c[!] Task cannot be empty!");
                return;
            }
            const isCompleted = response.formValues[1];
            if (status != isCompleted)
                changed = true;

            updateGlobalList(GLOBALLIST, player, task, index, isCompleted, changed);

            updateBookLore(player, task, index, isCompleted, changed);

            if (task) {
                player.sendMessage(`Task updated successfully: §a§l${task}`);
                show_todo_list(player);
            }
        });
}


function delete_task(player, index, task, status, GLOBALLIST) {
    let state = (status) ? "Completed" : "Pending";
    const messageForm = new MessageFormData()
        .title(`§l§7Task ${index + 1} of ${state} Tasks`)
        .body(`§lTask :§r${task}\n§lAction : §rTask deleted successfully!`)
        .button1("Done")
        .button2("Restore Task");

    messageForm
        .show(player)
        .then((formData) => {
            if (formData.canceled || formData.selection === undefined) {
                removeFromGlobalList(GLOBALLIST, player, index, status);
                deleteBookLore(player, task, index, status);
                return;
            }
            if (formData.selection == 1)
                return;
            else if (formData.selection == 0) {
                removeFromGlobalList(GLOBALLIST, player, index, status);
                deleteBookLore(player, task, index, status);
                player.sendMessage("§l§c[!] Task deleted successfully!");
            }
        })
}
